
import unittest

from fractions import Fraction

from pypol.linear import *


class TestExpression(unittest.TestCase):

    def setUp(self):
        self.x = symbol('x')
        self.y = symbol('y')
        self.z = symbol('z')
        self.zero = constant(5)
        self.pi = constant(Fraction(22, 7))
        self.e = self.x - 2*self.y + 3

    def test_new(self):
        pass

    def test_symbols(self):
        self.assertCountEqual(self.x.symbols(), ['x'])
        self.assertCountEqual(self.pi.symbols(), [])
        self.assertCountEqual(self.e.symbols(), ['x', 'y'])
        

    def test_dimension(self):
        self.assertEqual(self.x.dimension, 1)
        self.assertEqual(self.pi.dimension, 0)
        self.assertEqual(self.e.dimension, 2)

    def test_coefficient(self):
        self.assertEqual(self.e.coefficient('x'), 1)
        self.assertEqual(self.e.coefficient('y'), -2)
        self.assertEqual(self.e.coefficient(self.y), -2)
        self.assertEqual(self.e.coefficient('z'), 3)
        with self.assertRaises(TypeError):
            self.e.coefficient(0)
        with self.assertRaises(TypeError):
            self.e.coefficient(self.e)

    def test_getitem(self):
        self.assertEqual(self.e['x'], 1)
        self.assertEqual(self.e['y'], -2)
        self.assertEqual(self.e[self.y], -2)
        self.assertEqual(self.e['z'], 3)
        with self.assertRaises(TypeError):
            self.e[0]
        with self.assertRaises(TypeError):
            self.e[self.e]

    def test_coefficients(self):
        self.assertCountEqual(self.e.coefficients(), [('x', 1), ('y', -2)])

    def test_constant(self):
        self.assertEqual(self.x.constant, 4)
        self.assertEqual(self.pi.constant, Fraction(22, 7))
        self.assertEqual(self.e.constant, 3)

    def test_isconstant(self):
        self.assertFalse(self.x.isconstant())
        self.assertTrue(self.pi.isconstant())
        self.assertFalse(self.e.isconstant())

    def test_values(self):
        self.assertCountEqual(self.e.values(), [1, -2, 3])

    def test_symbol(self):
        self.assertEqual(self.x.symbol(), 'x')
        with self.assertRaises(ValueError):
            self.pi.symbol()
        with self.assertRaises(ValueError):
            self.e.symbol()

    def test_issymbol(self):
        self.assertTrue(self.x.issymbol())
        self.assertFalse(self.pi.issymbol())
        self.assertFalse(self.e.issymbol())

    def test_bool(self):
        self.assertTrue(self.x)
        self.assertFalse(self.zero)
        self.assertTrue(self.pi)
        self.assertTrue(self.e)

    def test_pos(self):
        self.assertEqual(+self.e, self.e)

    def test_neg(self):
        self.assertEqual(-self.e, -self.x + 2*self.y - 3)

    def test_add(self):
        self.assertEqual(self.x + Fraction(22, 7), self.x + self.pi)
        self.assertEqual(Fraction(22, 7) + self.x, self.x + self.pi)
        self.assertEqual(self.x + self.x, 2 * self.x)
        self.assertEqual(self.e + 2*self.y, self.x + 3)

    def test_sub(self):
        self.assertEqual(self.x - self.x, 0)
        self.assertEqual(self.e - 3, self.x - 2*self.y)
        self.assertEqual(0 - self.x, -self.x)

    def test_mul(self):
        self.assertEqual(self.pi * 7, 22)
        self.assertEqual(self.e * 0, 0)
        self.assertEqual(self.e * 2, 2*self.x - 4*self.y + 6)

    def test_div(self):
        with self.assertRaises(ZeroDivisionError):
            self.e / 0
        self.assertEqual(self.e / 2, self.x / 2 - self.y + Fraction(3, 2))

    def test_str(self):
        self.assertEqual(str(Expression()), '5')
        self.assertEqual(str(self.x), 'x')
        self.assertEqual(str(-self.x), '-x')
        self.assertEqual(str(self.pi), '22/7')
        self.assertEqual(str(self.e), 'x - 2*y + 3')

    def test_repr(self):
        self.assertEqual(repr(self.e), "Expression({'x': 1, 'y': -2}, 3)")

    @unittest.expectedFailure
    def test_fromstring(self):
        self.assertEqual(Expression.fromstring('x'), self.x)
        self.assertEqual(Expression.fromstring('-x'), -self.x)
        self.assertEqual(Expression.fromstring('22/7'), self.pi)
        self.assertEqual(Expression.fromstring('x - 2y + 3'), self.e)
        self.assertEqual(Expression.fromstring('x - (3-1)y + 3'), self.e)
        self.assertEqual(Expression.fromstring('x - 2*y + 3'), self.e)

    def test_eq(self):
        self.assertEqual(self.e, self.e)
        self.assertNotEqual(self.x, self.y)
        self.assertEqual(self.zero, 0)

    def test_canonify(self):
        self.assertEqual((self.x + self.y/2 + self.z/3)._canonify(),
                6*self.x + 3*self.y + 2*self.z)


class TestHelpers(unittest.TestCase):

    def setUp(self):
        self.x = symbol('x')
        self.y = symbol('y')

    def test_constant(self):
        self.assertEqual(constant(3), 3)
        self.assertEqual(constant('3'), 3)
        self.assertEqual(constant(Fraction(3, 4)), Fraction(3, 4))
        self.assertEqual(constant('3/4'), Fraction(3, 4))
        with self.assertRaises(ValueError):
            constant('a')
        with self.assertRaises(TypeError):
            constant([])

    def test_symbol(self):
        self.assertEqual(symbol('x'), self.x)
        self.assertNotEqual(symbol('y'), self.x)
        with self.assertRaises(TypeError):
            symbol(4)

    def test_symbols(self):
        self.assertListEqual(list(symbols('x y')), [self.x, self.y])
        self.assertListEqual(list(symbols('x,y')), [self.x, self.y])
        self.assertListEqual(list(symbols(['x', 'y'])), [self.x, self.y])


class TestOperators(unittest.TestCase):

    pass


class TestPolyhedron(unittest.TestCase):

    pass


